#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-10-04
# File  : wato/active_checks_radius.py
#
# 2024-01-01: modified for cmk 2.2.x

from cmk.gui.i18n import _
from cmk.gui.plugins.wato.active_checks.common import RulespecGroupActiveChecks
from cmk.gui.plugins.wato.utils import HostRulespec, IndividualOrStoredPassword, rulespec_registry
from cmk.gui.valuespec import (
    Alternative,
    Dictionary,
    DropdownChoice,
    FixedValue,
    Foldable,
    IPv4Address,
    Integer,
    ListOf,
    MonitoringState,
    TextInput,
    Transform,
    Tuple,
)

_called_station_id = Tuple(
    title='Called-Station-Id',
    orientation='horizontal',
    elements=[
        FixedValue('Called-Station-Id'),  # , totext='' # add empty totext to remove "duplicate" attribute name
        TextInput(
            size=20,
            placeholder='AA-BB-CC-DD-EE-FF',
            allow_empty=False,
        ),
    ],
)
_calling_station_id = Tuple(
    title='Calling-Station-Id',
    orientation='horizontal',
    elements=[
        FixedValue('Calling-Station-Id'),
        TextInput(
            size=20,
            placeholder='AA-BB-CC-DD-EE-FF',
            allow_empty=False,
        ),
    ],
)
_framed_mtu = Tuple(
    title='Framed-MTU',
    orientation='horizontal',
    elements=[
        FixedValue('Framed-MTU'),
        Integer(
            size=5,
            default_value=1500
        ),
    ],
)
_nas_identifier = Tuple(
    title='NAS-Identifier',
    orientation='horizontal',
    elements=[
        FixedValue('NAS-Identifier'),
        TextInput(
            size=20,
            placeholder='NAS001',
            allow_empty=False,
        ),
    ],
)
_nas_ip_address = Tuple(
    title='NAS-IP-Address',
    orientation='horizontal',
    elements=[
        FixedValue('NAS-IP-Address'),
        IPv4Address(),
    ],
)
_nas_port_id = Tuple(
    title='NAS-Port-Id',
    orientation='horizontal',
    elements=[
        FixedValue('NAS-Port-Id'),
        TextInput(
            size=20,
            placeholder='GigabitEthernet0/8',
            allow_empty=False,
        ),
    ],
)
_nas_port_type = Tuple(
    title='NAS-Port-Type',
    orientation='horizontal',
    elements=[
        FixedValue('NAS-Port-Type'),
        DropdownChoice(
            choices=[
                ('0', 'Async'),
                ('1', 'Sync'),
                ('2', 'ISDN'),
                ('3', 'ISDN-V120'),
                ('4', 'ISDN-V110'),
                ('5', 'Virtual'),
                ('6', 'PIAFS'),
                ('7', 'HDLC-Clear-Channel'),
                ('8', 'X.25'),
                ('9', 'X.75'),
                ('10', 'G.3-Fax'),
                ('11', 'SDSL'),
                ('12', 'ADSL-CAP'),
                ('13', 'ADSL-DMT'),
                ('14', 'IDSL'),
                ('15', 'Ethernet'),
                ('16', 'xDSL'),
                ('17', 'Cable'),
                ('18', 'Wireless-Other'),
                ('19', 'Wireless-802.11'),
            ]
        ),
    ],
)
_nas_port = Tuple(
    title='NAS-Port',
    orientation='horizontal',
    elements=[
        FixedValue('NAS-Port'),
        Integer(
            size=7,
        ),
    ],
)
_service_type = Tuple(
    title='Service-Type',
    orientation='horizontal',
    elements=[
        FixedValue('Service-Type'),
        DropdownChoice(
            choices=[
                ('1', 'Login-User'),
                ('2', 'Framed-User'),
                ('3', 'Callback-Login-User'),
                ('4', 'Callback-Framed-User'),
                ('5', 'Outbound-User'),
                ('6', 'Administrative-User'),
                ('7', 'NAS-Prompt-User'),
                ('8', 'Authenticate-Only'),
                ('9', 'Callback-NAS-Prompt'),
                ('10', 'Call-Check'),
                ('11', 'Callback-Administrative'),
            ]
        ),
    ],
)


def _valuespec_active_checks_radius():
    return Transform(
        Dictionary(
            title=_('Check RADIUS service'),
            help=_(''),
            elements=[
                ('description',
                 TextInput(
                     title=_('Service description'),
                     help=_(
                         'Must be unique for every host. The service description starts always with \"RADIUS server\".'
                     ),
                     size=50,
                     placeholder='Item name for the service',
                     allow_empty=False,
                 )),
                ('server',
                 TextInput(
                     title=_('Server IP-address or name'),
                     help=_(
                         'Hostname or IP-address to monitor. Default is the host name/IP-Address of the monitored host.'
                     ),
                     size=50,
                     allow_empty=False,
                     placeholder='i.e 192.168.10.10 or srvrad01.company.intern'
                 )),
                ('auth_port',
                 Integer(
                     title=_('Authentication port'),
                     help=_('The RADIUS port to use for authentication. Default is 1812.'),
                     size=5,
                     default_value=1812,
                     minvalue=1,
                     maxvalue=65535,
                 )),
                ('secret',
                 IndividualOrStoredPassword(
                     title=_('Shared secret'),
                     help=_('The shared secret.'),
                     allow_empty=False,
                 )),
                ('timeout',
                 Integer(
                     title=_('Request timeout'),
                     help=_('The timeout for the RADIUS request.'),
                     default_value=2,
                     minvalue=1,
                     maxvalue=30,
                     unit='s',
                 )),
                ('user_name',
                 TextInput(
                     title=_('User name'),
                     help=_('The user name to use in the request.'),
                     size=50,
                     placeholder='user name to use in the request',
                     allow_empty=False,
                 )),
                ('user_password',
                 IndividualOrStoredPassword(
                     title=_('User password'),
                     help=_('The user password.'),
                     allow_empty=False
                 )),
                ('request_params',
                 Foldable(
                     Dictionary(
                         title="Hide/Show Request Parameters",
                         elements=[
                             ('request_attributes',
                              ListOf(
                                  Alternative(
                                      orientation='horizontal',
                                      elements=[
                                          _called_station_id,
                                          _calling_station_id,
                                          _framed_mtu,
                                          _nas_identifier,
                                          _nas_ip_address,
                                          _nas_port,
                                          _nas_port_id,
                                          _nas_port_type,
                                          _service_type,
                                      ],
                                  ),
                                  title=_('Request Attributes'),
                                  add_label=_('add attribute'),
                              )),
                         ],
                     ),
                     title='Request paranmeters',
                 )),
                ('response_params',
                 Foldable(
                     Dictionary(
                         title="Hide/Show Response Parameters",
                         elements=[
                             ('expected_response',
                              DropdownChoice(
                                  choices=(
                                      (2, 'Accepted'),
                                      (3, 'Rejected'),
                                  ),
                                  title=_('Expected response'),
                                  help=_('Expected response from the RADIUS server.'),
                              )),
                             ('state_not_expected_response',
                              MonitoringState(
                                  title=_('Monitoring state not expected response'),
                                  default_value=2,
                              )),
                             ('level_upper_response_time',
                              Tuple(
                                  elements=[
                                      Integer(title=_('Warning at'), unit='ms', minvalue=0, maxvalue=10000),
                                      Integer(title=_('Critical at'), unit='ms', minvalue=0, maxvalue=10000),
                                  ],
                                  title=_('Max. response time'),
                              )),
                             ('num_resp_attributes',
                              Integer(
                                  title=_('# of expected attribues in response'),
                                  help=_('The expected number of RADIUS attibutes in the response.'),
                                  minvalue=0,
                                  maxvalue=65535,
                              )),
                             ('state_wrong_number_of_response_attributes',
                              MonitoringState(
                                  title=_('Monitoring state on wrong # of response attributes'),
                                  default_value=1,
                              )),
                         ],
                     ),
                     title='Response paranmeters',
                 ))
            ],
            required_keys=[
                'secret',
                'request_params',
                'response_params',
            ]
        ),
    )


rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupActiveChecks,
        match_type='all',
        name='active_checks:radius',
        valuespec=_valuespec_active_checks_radius,
    )
)
